package gssim.gssimstudio.editor.gssim.topology.file
{
	import flash.events.Event;
	import flash.geom.Utils3D;
	import flash.net.FileReference;
	import flash.utils.ByteArray;
	import flash.utils.Dictionary;
	
	import gssim.gssimstudio.editor.flare.util.Arrays;
	import gssim.gssimstudio.editor.gssim.editors.topology.ObjectCanvas;
	import gssim.gssimstudio.editor.gssim.topology.GenderObject;
	import gssim.gssimstudio.editor.gssim.topology.GenderObjectFactory;
	import gssim.gssimstudio.editor.gssim.topology.GraphItem;
	import gssim.gssimstudio.editor.gssim.topology.ResourceObject;
	import gssim.gssimstudio.editor.gssim.topology.RouterObject;
	import gssim.gssimstudio.editor.gssim.topology.UserObject;
	import gssim.gssimstudio.editor.gssim.topology.layouts.gssimRadialLayout;
	import gssim.gssimstudio.editor.script.event.ObjectEvent;
	
	import mx.controls.Alert;
	import mx.core.ByteArrayAsset;
	import mx.utils.ArrayUtil;
	import mx.utils.StringUtil;
	
	/**
	 * Loads network topology from file
	 */
	public class topologyReader
	{
		private var fref : FileReference;
		private var _canvas : ObjectCanvas;
		
		public static const PHASE_ROUTERS:String = "routers";
		public static const PHASE_ROUTERS_LINKS:String = "routers_links";
    	public static const PHASE_RESOURCES_LINKS:String = "resources_links";
    	public static const PHASE_USER_LINKS:String = "user_links";
    	public static const PHASE_END:String = "eof";
		
		public function topologyReader(canvas : ObjectCanvas, autobrowse : Boolean = true)
		{
			_canvas = canvas;
			if (autobrowse)
			{
				fref = new FileReference();
				fref.addEventListener(Event.SELECT, selectHandler);
				fref.addEventListener(Event.COMPLETE, completeHandler);
				fref.browse();
			}
		}
		
		public function selectHandler(event:Event):void
		{
		    fref.load();
		}
			
		public function readFromFile(file : FileReference) : void
		{
			fref = file;
			completeHandler(null);
		}
		
		public function readFromBinary(data : ByteArray) : void
		{
			var copy : ByteArray = new ByteArray();
			copy.writeObject(data);
			copy.position = 0;
			copy = copy.readObject() as ByteArray;
			var text : String = copy.readUTFBytes(copy.bytesAvailable);
			var array : Array;
			//will this be the same on unix ?
			array = text.split('\r\n');
			if (array.length == 1) array = text.split('\r');
			if (array.length == 1) array = text.split('\n');
			if (array.length == 1)
			{
				Alert.show("Inproper file format!");
				return;
			}
			for (var i : int = 0; i < array.length; i++)
			{
				array[i] = StringUtil.trim(array[i]);
			}
			
			readFromString(array);
		}
		
		public function completeHandler(event:Event):void
		{
			if (_canvas == null)
			{
				Alert.show("Missing canvas!");
				return;
			}
			
			if (fref == null)
			{
				Alert.show("Missing file!");
				return;
			}
			
		    var data:ByteArray = fref.data;
			
			readFromBinary(data);
			
			fref = null;
		}
		
		public function readFromString(array : Array) : void
		{
			_canvas.empty();
			var count : int = -1;
			var phase : String = PHASE_ROUTERS;
			var usedNames : Dictionary = new Dictionary();
			var splitted : Array;
			var name1 : String;
			var name2 : String;
			var baudRate : Number;
			var propDelay : Number;
			var mtu : Number;
			var nodeX : Number;
			var nodeY : Number;
	
			//read file content into ObjectCanvas
			for (var i : int = 0; i < array.length; i++)
			{
				if (array[i].charAt(0) == '#' || array[i] == "")
					continue;
				if (phase == PHASE_ROUTERS)
				{ 
					if (count == -1)
						count = parseInt(array[i]);
					else
					if (count == 0)
					{
						phase = PHASE_ROUTERS_LINKS;
						count = -1;
					}
					else
					{
						count--;
						var router : RouterObject = GenderObjectFactory.createRouter(
								0,
								0,
								94,
								94);
						router.nodeName = StringUtil.trim(array[i]);
						router.id = router.nodeName;
						_canvas.addGenderObject(router);
					}
				}
				if (phase == PHASE_ROUTERS_LINKS)
				{ 
					if (count == -1)
						count = parseInt(array[i]);
					else
					if (count == 0)
					{
						phase = PHASE_RESOURCES_LINKS;
						count = -1;
					}
					else
					{
						count--;
						splitted = array[i].replace(/\s+/g, ' ').split(' ');
						name1 = splitted[0];
						name2 = splitted[1];
						baudRate = Number(splitted[2]);
						propDelay = Number(splitted[3]);
						mtu = Number(splitted[4]);
						_canvas.linkObjectsById(name1, name2, baudRate, propDelay, mtu);
					}
				}
				if (phase == PHASE_RESOURCES_LINKS)
				{ 
					if (count == -1)
						count = parseInt(array[i]);
					else
					if (count == 0)
					{
						phase = PHASE_USER_LINKS;
						count = -1;
					}
					else
					{
						count--;
						splitted = array[i].replace(/\s+/g, ' ').split(' ');
						name1 = splitted[0];
						name2 = splitted[1];
						if (!usedNames.hasOwnProperty(name1))
						{
							var resource : ResourceObject = GenderObjectFactory.createResource(
									0,
									0,
									94,
									94);
							resource.nodeName = name1;
							resource.id = resource.nodeName;
							usedNames[name1] = "1";
							_canvas.addGenderObject(resource);
						}
						baudRate = Number(splitted[2]);
						propDelay = Number(splitted[3]);
						mtu = Number(splitted[4]);
						_canvas.linkObjectsById(name1, name2, baudRate, propDelay, mtu);
					}
				}
				if (phase == PHASE_USER_LINKS)
				{ 
					if (count == -1)
					{
						count = parseInt(array[i]);
						usedNames = new Dictionary();
					}
					else
					if (count == 0)
					{
						phase = PHASE_END;
						_canvas.forceStopDrag(null);
						_canvas.applyLayout(new gssimRadialLayout());
						count = -1;
					}
					else
					{
						count--;
						splitted = array[i].replace(/\s+/g, ' ').split(' ');
						name1 = splitted[0];
						name2 = splitted[1];
						if (!usedNames.hasOwnProperty(name1))
						{
							var user : UserObject = GenderObjectFactory.createUser(
									0,
									0,
									94,
									94);
							user.nodeName = name1;
							user.id = user.nodeName;
							usedNames[name1] = "1";
							_canvas.addGenderObject(user);
						}
						baudRate = Number(splitted[2]);
						propDelay = Number(splitted[3]);
						mtu = Number(splitted[4]);
						_canvas.linkObjectsById(name1, name2, baudRate, propDelay, mtu);
					}
				}
				if (phase == PHASE_END)
				{ 
					if (count == -1)
					{
						count = parseInt(array[i]);
					}
					else
					if (count == 0)
					{
						phase = PHASE_END;
						count = -1;
					}
					else
					{
						count--;
						splitted = array[i].replace(/\s+/g, ' ').split(' ');
						name1 = splitted[0];
						nodeX = Number(splitted[1]);
						nodeY = Number(splitted[2]);
						var temp : GraphItem = ObjectCanvas.graph.find(name1);
						if (temp != null)
						{
							temp.x = nodeX;
							temp.y = nodeY;
							temp.dispatchMoveAction();
							_canvas.dispatchEvent(new ObjectEvent(ObjectCanvas.OBJECT_MOVED, temp as GenderObject));
						}
					}
				}
			}
			if (count == 0 && phase == PHASE_USER_LINKS)
			{
				phase = PHASE_END;
				_canvas.forceStopDrag(null);
				_canvas.applyLayout(new gssimRadialLayout());
				count = -1;
			}
			_canvas = null;
		}

	}
}
